#!/usr/bin/env python

import serial
import time
import sys
import glob
import argparse
from collections import deque

try:
  import numpy as np
  import matplotlib.pyplot as plt
  import matplotlib.pyplot as plt
  import matplotlib.animation as anim
except ImportError:
  print "Please check your Anaconda installation. This program requires both numpy and matplotlib."
  exit()

# Based on tutorial
# http://josephmr.com/realtime-arduino-sensor-monitoring-with-matplotlib/

MAX_X = 100   # Width of graph
MAX_Y = 5  # Height of
line = deque([0.0]*MAX_X, maxlen=MAX_X)

# Serial functions
def serial_ports():
    """Lists serial ports

    Raises:
    EnvironmentError:
        On unsupported or unknown platforms
    Returns:
        A list of available serial ports
    """
    if sys.platform.startswith('win'):
        ports = ['COM' + str(i + 1) for i in range(256)]

    elif sys.platform.startswith('linux') or sys.platform.startswith('cygwin'):
        # this is to exclude your current terminal "/dev/tty"
        ports = glob.glob('/dev/tty[A-Za-z]*')

    elif sys.platform.startswith('darwin'):
        ports = glob.glob('/dev/tty.*')

    else:
        raise EnvironmentError('Unsupported platform')

    result = []
    for port in ports:
        try:
            s = serial.Serial(port)
            s.close()
            result.append(port)
        except (OSError, serial.SerialException):
            pass
    return result

def open_port(port, baud):
    """Open a serial port.

    Args:
    port (string): port to open, on Unix typically begin with '/dev/tty', on
        or 'COM#' on Windows.
    baud (int, optional): baud rate for serial communication

    Raises:
    SerialException: raised for errors from serial communication

    Returns:
       A pySerial object to communicate with the serial port.
    """
    ser = serial.Serial()
    try:
        ser = serial.Serial(port, baud, timeout=10)
        print "Opened serial connection on port %s"%port
        return ser
    except serial.SerialException:
        raise

def get_port():
  ports = serial_ports()
  if len(ports) > 1:
    for p in ports:
      if p != "COM1":
        return p
  else:
    print "No serial ports detected. Connect the Arduino and try again."

def transmit(binary, device="/dev/ttyACM0", baud=9600):
    ser = serial.Serial()
    try:
        ser = open_port(device,baud)
    except:
        print "Cannot open port %s"%device
        quit()

    ser.readline()
    print "Setting binary code to transmit: %s"%binary
    ser.write(binary)
    print ser.readline()

if __name__ == "__main__":
  # Parse arguments
  parser = argparse.ArgumentParser(description='Transmit to a wireless light switch.')
  parser.add_argument('binary')
  parser.add_argument('-D', '--device', choices=serial_ports(),
          help='a serial port to read from')
  parser.add_argument('-b', '--baud', type=int, default=9600,
          help="baud rate for the serial connection ")
  parser.add_argument('--debug', action='store_true', default=False,
          help='print all raw ADC values')
  args = parser.parse_args()

  port = get_port()
  if port:
      correct = raw_input("Port %s selected, is this correct? (y/n) "%port).lower()
      if correct == 'y':
        transmit(args.binary, port, args.baud)
      else:
        print "Check Arduino connection"
